# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import wx, const, copy, os, gettext
from Components import *

def _(x):
	return gettext.gettext(x).decode("utf-8")

class CircuitManager():
	def __init__(self, parent, *args, **kwds):
		self.parent = parent
		self.components = []
		self.selected_components = []
		self.net_connections = []
		self.net_levels = []
		self.net_no_dot = []
		self.components_history = [[]]
		self.action_count = 0
		self.save_point = 0
		self.need_save = False
		self.simple_change = True
		self.filepath = ""
		
	def save_file(self, filepath):
		try:
			fp = open(filepath, mode="w")
		except IOError:
			return True
		for c in self.components:
			if c[0] == const.component_net:
				fp.write("net\n position=%d,%d,%d,%d\n\n" % (c[1], c[2], c[3], c[4]))
			else:
				fp.write("%s\n" % c[0])
				fp.write(" position=%d,%d\n" % (c[1].pos_x, c[1].pos_y))
				fp.write(" input_level=%s\n" % ",".join([str(int(p)) for p in c[1].input_level]))
				fp.write(" output_level=%s\n\n" % ",".join([str(int(p)) for p in c[1].output_level]))
				
		fp.close()
		self.filepath = filepath
		self.need_save = False
		self.simple_change = True
		self.save_point = self.action_count
		self.parent.SetTitle("%s - wxLogic" % os.path.basename(filepath))
		return False
	
	def open_file(self, filepath):
		try:
			fp = open(filepath, mode="r")
		except IOError:
			return True
		self.reset_circuit()
		component = None
		for l in fp:
			l = l.strip()
			if l == "net":
				component = [const.component_net, 0, 10, 10, 10]
			elif l in const.comp_dict:
				component = [l, copy.deepcopy(const.comp_dict[l])]
			elif component is not None:
				if l == "":
					self.components.append(component)
					component = None
					
				else:
					prop = l.split("=")
					if len(prop) == 2:
						if prop[0] == "position":
							p = prop[1].split(",")
							if component[0] == const.component_net:
								if len(p) == 4:
									component[1] = int(p[0])
									component[2] = int(p[1])
									component[3] = int(p[2])
									component[4] = int(p[3])
							else:
								if len(p) == 2:
									component[1].pos_x = int(p[0])
									component[1].pos_y = int(p[1])
						
						elif prop[0] == "input_level":
							if component[0] != const.component_net:
								p = prop[1].split(",")
								if len(component[1].input_level) == len(p):
									component[1].input_level = [bool(int(s if s.isdigit() else 0)) for s in p]
									
						elif prop[0] == "output_level":
							if component[0] != const.component_net:
								p = prop[1].split(",")
								if len(component[1].output_level) == len(p):
									component[1].output_level = [bool(int(s if s.isdigit() else 0)) for s in p]
			
		fp.close()
		self.components_history = [copy.deepcopy(self.components)]
		self.filepath = filepath
		self.parent.SetTitle("%s - wxLogic" % os.path.basename(filepath))
		return False
		
	def analyze_connections(self):
		# Analyze connections
		self.net_connections = []
		self.net_no_dot = []
		not_added = self.components[:]
		net_not_empty = True
		while True:
			connection = []
			changed_flag = True
			net_not_empty = False
			while changed_flag:
				changed_flag = False
				remlist = []
				for c in not_added:
					if c[0] == const.component_net:
						net_not_empty = True
						if len(connection) == 0:
							connection.append((c[1], c[2]))
							connection.append((c[3], c[4]))
							remlist.append(c)
						else:
							if (c[1], c[2]) in connection:
								if not (c[3], c[4]) in connection:
									connection.append((c[3], c[4]))
									changed_flag = True
								remlist.append(c)
							elif (c[3], c[4]) in connection:
								if not (c[1], c[2]) in connection:
									connection.append((c[1], c[2]))
									changed_flag = True
								remlist.append(c)
				for c in remlist:
					not_added.remove(c)
					
			if not net_not_empty:
				break
				
			for con_p in connection:
				cnt = 0
				for c in self.components:
					if c[0] == const.component_net:
						if con_p[0] == c[1] and con_p[1] == c[2]:
							cnt += 1
						elif con_p[0] == c[3] and con_p[1] == c[4]:
							cnt += 1
					else:
						for p in c[1].input_pins + c[1].output_pins:
							if con_p[0] == c[1].pos_x + p[0] and con_p[1] == c[1].pos_y + p[1]:
								cnt += 1
				if cnt <= 2:
					self.net_no_dot.append(con_p)
				
			self.net_connections.append(connection)
			
		for c in self.components:
			if c[0] != const.component_net:
				for p in c[1].input_pins + c[1].output_pins:
					found = False
					for net in self.net_connections:
						if (c[1].pos_x + p[0], c[1].pos_y + p[1]) in net:
							found = True
							break
					if not found:
						self.net_connections.append([(c[1].pos_x + p[0], c[1].pos_y + p[1])])
			
	def split_nets(self, x, y):
		# Split nets on the specified point
		spl_list = []
		for c in self.components:
			if c[0] == const.component_net:
				if (y - c[2]) * (c[3] - c[1]) == (x - c[1]) * (c[4] - c[2]):
					if (c[1] <= x <= c[3] and c[2] <= y <= c[4]) or (c[3] <= x <= c[1] and c[4] <= y <= c[2]):
						if (x != c[1] or y != c[2]) and (x != c[3] or y != c[4]):
							spl_list.append(c)
							
		for c in spl_list:
			self.components.remove(c)
			net_selected = False
			if c in self.selected_components:
				self.selected_components.remove(c)
				net_selected = True
			if not [const.component_net, c[1], c[2], x, y] in self.components and not [const.component_net, x, y, c[1], c[2]] in self.components:
				component_data = [const.component_net, c[1], c[2], x, y]
				self.components.append(component_data)
				if net_selected:
					self.selected_components.append(component_data)
			if not [const.component_net, c[3], c[4], x, y] in self.components and not [const.component_net, x, y, c[3], c[4]] in self.components:
				component_data = [const.component_net, x, y, c[3], c[4]]
				self.components.append(component_data)
				if net_selected:
					self.selected_components.append(component_data)
	
	def connect_nets(self, x, y, lock_selected = False):
		nets = []
		for c in self.components:
			if c[0] == const.component_net:
				if c[1] == x and c[2] == y or c[3] == x and c[4] == y:
					if not (lock_selected and c in self.selected_components):
						nets.append(c)
			else:
				# Cannot connect if there are terminals of components
				for p in c[1].input_pins + c[1].output_pins:
					if c[1].pos_x + p[0] == x and c[1].pos_y + p[1] == y:
						return
		
		if len(nets) == 2:
			if (nets[0][1] - nets[0][3]) * (nets[1][2] - nets[1][4]) == (nets[1][1] - nets[1][3]) * (nets[0][2] - nets[0][4]):
				if nets[0][1] == x and nets[0][2] == y:
					if nets[1][1] == x and nets[1][2] == y:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) <= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) <= 0:
							component_data = [const.component_net, nets[0][3], nets[0][4], nets[1][3], nets[1][4]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)
					else:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) >= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) >= 0:
							component_data = [const.component_net, nets[0][3], nets[0][4], nets[1][1], nets[1][2]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)
				else:
					if nets[1][1] == x and nets[1][2] == y:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) >= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) >= 0:
							component_data = [const.component_net, nets[0][1], nets[0][2], nets[1][3], nets[1][4]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)
					else:
						if (nets[0][1] - nets[0][3]) * (nets[1][1] - nets[1][3]) <= 0 and (nets[0][2] - nets[0][4]) * (nets[1][2] - nets[1][4]) <= 0:
							component_data = [const.component_net, nets[0][1], nets[0][2], nets[1][1], nets[1][2]]
							self.components.append(component_data)
							self.components.remove(nets[0])
							self.components.remove(nets[1])
							net_selected = False
							if nets[0] in self.selected_components:
								self.selected_components.remove(nets[0])
								net_selected = True
							if nets[1] in self.selected_components:
								self.selected_components.remove(nets[1])
								net_selected = True
							if net_selected:
								self.selected_components.append(component_data)
							
	def analyze_logic(self):
		net_levels_history = []
		counter = 0
		osc = False
		while True:
			self.net_levels = [-1]*len(self.net_connections)
			for c in self.components:
				if c[0] != const.component_net:
					for i,p in enumerate(c[1].output_pins):
						for j,net in enumerate(self.net_connections):
							if (c[1].pos_x + p[0], c[1].pos_y + p[1]) in net:
								if self.net_levels[j] != -1 and self.net_levels[j] != c[1].output_level[i]:
									self.parent.set_status(_("Output port is short circuit!"))
									return True
								self.net_levels[j] = c[1].output_level[i]
								
			for c in self.components:
				if c[0] != const.component_net:
					input_datas = []
					for i,p in enumerate(c[1].input_pins):
						for j,net in enumerate(self.net_connections):
							if (c[1].pos_x + p[0], c[1].pos_y + p[1]) in net:
								if self.net_levels[j] == -1:
									self.parent.set_status(_("Input port is open circuit!"))
									return True
								input_datas.append(self.net_levels[j])
					if c[1].calculate(input_datas):
						self.parent.set_status(_("Input is undefined!"))
						return True
			
			if self.net_levels in net_levels_history:
				if len(net_levels_history) != 0:
					if self.net_levels == net_levels_history[len(net_levels_history) - 1]:
						self.parent.set_status("")
						return False
					else:
						if osc:
							self.parent.set_status(_("This circuit ocillated!"))
							return True
							
						osc = True
			net_levels_history.append(self.net_levels)
			counter += 1
			if counter >= 1000:
				self.parent.set_status(_("This logic is complexity!"))
				return True
	
	def remove_selected_component(self):
		for c in self.selected_components:
			self.components.remove(c)
			
		for c in self.selected_components:
			if c[0] == const.component_net:
				self.connect_nets(c[1], c[2])
				self.connect_nets(c[3], c[4])
			else:
				for p in c[1].input_pins + c[1].output_pins:
					self.connect_nets(c[1].pos_x + p[0], c[1].pos_y + p[1])
					
		self.selected_components = []
	
	def reset_circuit(self):
		self.selected_components = []
		self.components = []
		self.components_history = [[]]
		self.action_count = 0
		self.save_point = 0
		self.need_save = False
		self.simple_change = True
		self.filepath = ""
		
	def push_history(self):
		self.action_count += 1
		if self.action_count < len(self.components_history):
			self.components_history = self.components_history[0:self.action_count]
			if self.action_count <= self.save_point:
				self.simple_change = False
		self.components_history.append(copy.deepcopy(self.components))
		self.need_save = True
		self.parent.SetTitle(u"%s [%s] - wxLogic" % (os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle, const.text_modified))
		
	def undo(self):
		self.action_count -= 1
		if self.action_count >= 0:
			self.components = copy.deepcopy(self.components_history[self.action_count])
			
		if self.save_point == self.action_count and self.simple_change:
			self.need_save = False
			self.parent.SetTitle(u"%s - wxLogic" % (os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle))
		else:
			self.need_save = True
			self.parent.SetTitle(u"%s [%s] - wxLogic" % ((os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle), const.text_modified))
		
		self.selected_components = []
		
	def redo(self):
		self.action_count += 1
		if self.action_count < len(self.components_history):
			self.components = copy.deepcopy(self.components_history[self.action_count])
			
		if self.save_point == self.action_count and self.simple_change:
			self.need_save = False
			self.parent.SetTitle(u"%s - wxLogic" % (os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle))
		else:
			self.need_save = True
			self.parent.SetTitle(u"%s [%s] - wxLogic" % ((os.path.basename(self.filepath) if self.filepath != "" else const.text_notitle), const.text_modified))
		
		self.selected_components = []
		c[1].output_pins
