# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import wx, gettext
import wx.lib.buttons
import config, const
from functools import partial

def _(x):
	return gettext.gettext(x).decode("utf-8")

class ComponentWindow(wx.Frame):
	def __init__(self, parent, *args, **kwds):
		kwds["style"] = (wx.DEFAULT_FRAME_STYLE | wx.FRAME_TOOL_WINDOW | wx.FRAME_FLOAT_ON_PARENT) & ~(wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX)
		wx.Frame.__init__(self, parent, size = (160, 160), *args, **kwds)
		self.parent = parent
		self.SetTitle(_("Components"))
		
		# Create toggle buttons
		self.logic_NOT = wx.lib.buttons.GenBitmapToggleButton(self, 1, wx.Bitmap(config.DATADIR+"/images/logic/not.png"))
		self.logic_AND = wx.lib.buttons.GenBitmapToggleButton(self, 2, wx.Bitmap(config.DATADIR+"/images/logic/and.png"))
		self.logic_OR = wx.lib.buttons.GenBitmapToggleButton(self, 3, wx.Bitmap(config.DATADIR+"/images/logic/or.png"))
		self.logic_XOR = wx.lib.buttons.GenBitmapToggleButton(self, 4, wx.Bitmap(config.DATADIR+"/images/logic/xor.png"))
		self.logic_NAND = wx.lib.buttons.GenBitmapToggleButton(self, 5, wx.Bitmap(config.DATADIR+"/images/logic/nand.png"))
		self.logic_NOR = wx.lib.buttons.GenBitmapToggleButton(self, 6, wx.Bitmap(config.DATADIR+"/images/logic/nor.png"))
		self.logic_RSFF = wx.lib.buttons.GenBitmapToggleButton(self, 7, wx.Bitmap(config.DATADIR+"/images/logic/rsff.png"))
		self.logic_JKFF = wx.lib.buttons.GenBitmapToggleButton(self, 8, wx.Bitmap(config.DATADIR+"/images/logic/jkff.png"))
		self.logic_DFF = wx.lib.buttons.GenBitmapToggleButton(self, 9, wx.Bitmap(config.DATADIR+"/images/logic/dff.png"))
		self.logic_TFF = wx.lib.buttons.GenBitmapToggleButton(self, 10, wx.Bitmap(config.DATADIR+"/images/logic/tff.png"))
		self.logic_counter = wx.lib.buttons.GenBitmapToggleButton(self, 11, wx.Bitmap(config.DATADIR+"/images/logic/counter.png"))
		self.logic_SW = wx.lib.buttons.GenBitmapToggleButton(self, 12, wx.Bitmap(config.DATADIR+"/images/logic/sw.png"))
		self.logic_7seg = wx.lib.buttons.GenBitmapToggleButton(self, 13, wx.Bitmap(config.DATADIR+"/images/logic/7seg.png"))
		self.logic_LED = wx.lib.buttons.GenBitmapToggleButton(self, 14, wx.Bitmap(config.DATADIR+"/images/logic/led.png"))
		self.logic_VDD = wx.lib.buttons.GenBitmapToggleButton(self, 15, wx.Bitmap(config.DATADIR+"/images/logic/vdd.png"))
		self.logic_GND = wx.lib.buttons.GenBitmapToggleButton(self, 16, wx.Bitmap(config.DATADIR+"/images/logic/gnd.png"))
		
		layout = wx.GridSizer(4, 4)
		layout.Add(self.logic_NOT)
		layout.Add(self.logic_AND)
		layout.Add(self.logic_OR)
		layout.Add(self.logic_XOR)
		layout.Add(self.logic_NAND)
		layout.Add(self.logic_NOR)
		layout.Add(self.logic_RSFF)
		layout.Add(self.logic_JKFF)
		layout.Add(self.logic_DFF)
		layout.Add(self.logic_TFF)
		layout.Add(self.logic_counter)
		layout.Add(self.logic_SW)
		layout.Add(self.logic_7seg)
		layout.Add(self.logic_LED)
		layout.Add(self.logic_VDD)
		layout.Add(self.logic_GND)

		self.SetSizer(layout)
		
		# Bind events
		self.logic_NOT.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_NOT, comp_name=const.component_NOT))
		self.logic_AND.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_AND, comp_name=const.component_AND))
		self.logic_OR.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_OR, comp_name=const.component_OR))
		self.logic_XOR.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_XOR, comp_name=const.component_XOR))
		self.logic_NAND.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_NAND, comp_name=const.component_NAND))
		self.logic_NOR.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_NOR, comp_name=const.component_NOR))
		self.logic_RSFF.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_RSFF, comp_name=const.component_RSFF))
		self.logic_JKFF.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_JKFF, comp_name=const.component_JKFF))
		self.logic_DFF.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_DFF, comp_name=const.component_DFF))
		self.logic_TFF.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_TFF, comp_name=const.component_TFF))
		self.logic_counter.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_counter, comp_name=const.component_counter))
		self.logic_SW.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_SW, comp_name=const.component_SW))
		self.logic_7seg.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_7seg, comp_name=const.component_7seg))
		self.logic_LED.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_LED, comp_name=const.component_LED))
		self.logic_VDD.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_VDD, comp_name=const.component_VDD))
		self.logic_GND.Bind(wx.EVT_BUTTON, partial(self.on_component_button_click, button=self.logic_GND, comp_name=const.component_GND))
		
		self.checked_button = const.component_none
		
	def on_component_button_click(self, event, button, comp_name):
		if event.GetIsDown():
			self.uncheck_all_buttons()
			button.SetToggle(True)
			self.checked_button = comp_name
			self.parent.drawarea.pushed_component = const.comp_dict[comp_name]
		else:
			self.checked_button = const.component_none
			self.parent.drawarea.pushed_component = const.comp_dict[const.component_none]
	
	def uncheck_all_buttons(self):
		self.parent.drawarea.netstarted = False
		self.logic_NOT.SetToggle(False)
		self.logic_AND.SetToggle(False)
		self.logic_OR.SetToggle(False)
		self.logic_XOR.SetToggle(False)
		self.logic_NAND.SetToggle(False)
		self.logic_NOR.SetToggle(False)
		self.logic_RSFF.SetToggle(False)
		self.logic_JKFF.SetToggle(False)
		self.logic_DFF.SetToggle(False)
		self.logic_TFF.SetToggle(False)
		self.logic_counter.SetToggle(False)
		self.logic_SW.SetToggle(False)
		self.logic_7seg.SetToggle(False)
		self.logic_LED.SetToggle(False)
		self.logic_VDD.SetToggle(False)
		self.logic_GND.SetToggle(False)
		self.parent.toolbar.ToggleTool(2, False)
		self.parent.menu_components.Check(False)
		self.parent.menu_net.Check(False)
		self.parent.menu_run.Check(False)
		
