# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import wx

class Logic_Component():
	def __init__(self, *args, **kwds):
		self.pos_x = 0
		self.pos_y = 0
		self.mouse_button = False
	
	def drawComponent(self, x, y, dc, adc, running_mode):
		return
	
	def isMouseOvered(self, x, y):
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		return False
		
	def mouse_down(self, x, y):
		if self.isMouseOvered(x, y):
			self.mouse_button = True
		
	def mouse_up(self, x, y):
		if self.mouse_button:
			self.mouse_button = False
			if self.isMouseOvered(x, y):
				return self.click(x, y)
		return False
		
	def click(self, x, y):
		return False
	
	def calculate(self, input_datas):
		return False

class Logic_Dummy(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = []
		self.output_pins = []
		self.input_level = []
		self.output_level = []
	
	def drawComponent(self, x, y, dc, adc, running_mode):
		return
	
	def isMouseOvered(self, x, y):
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		return False

class Logic_NOT(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -20)]
		self.output_pins = [(90, -20)]
		self.input_level = [False]
		self.output_level = [True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 20, x + 30, y - 20)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 72, y - 20, x + 90, y - 20)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 20, x + 30, y - 20)
			dc.DrawLine(x + 72, y - 20, x + 90, y - 20)
		dc.DrawLine(x + 30, y - 35, x + 30, y - 5)
		adc.DrawLine(x + 30, y - 35, x + 60, y - 20)
		adc.DrawLine(x + 30, y - 5, x + 60, y - 20)
		adc.DrawCircle(x + 66, y - 20, 6)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 87 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 87 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.output_level[0] = not input_datas[0]
		self.input_level[0] = input_datas[0]
		return False

class Logic_AND(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(100, -20)]
		self.input_level = [False, False]
		self.output_level = [False]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 80, y - 20, x + 100, y - 20)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			dc.DrawLine(x + 80, y - 20, x + 100, y - 20)
		dc.DrawLine(x + 30, y - 40, x + 30, y)
		adc.DrawLine(x + 30, y - 40, x + 60, y - 40)
		adc.DrawLine(x + 30, y, x + 60, y)
		adc.DrawArc(x + 60, y, x + 60, y - 40, x + 60, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 97 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 97 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.output_level[0] = input_datas[0] and input_datas[1]
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_OR(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(100, -20)]
		self.input_level = [False, False]
		self.output_level = [False]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 38, y - 30)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 38, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 80, y - 20, x + 100, y - 20)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 30, x + 38, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 38, y - 10)
			dc.DrawLine(x + 80, y - 20, x + 100, y - 20)
		adc.DrawSpline([(x + 30, y - 40), (x + 38, y - 30), (x + 40, y - 20), (x + 38, y - 10), (x + 30, y)])
		adc.DrawSpline([(x + 30, y - 40), (x + 40, y - 40), (x + 50, y - 39), (x + 60, y - 36), (x + 70, y - 30), (x + 80, y - 20)])
		adc.DrawSpline([(x + 30, y), (x + 40, y), (x + 50, y - 1), (x + 60, y - 4), (x + 70, y - 10), (x + 80, y - 20)])
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 97 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 97 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.output_level[0] = input_datas[0] or input_datas[1]
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_XOR(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(100, -20)]
		self.input_level = [False, False]
		self.output_level = [False]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 38, y - 30)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 38, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 80, y - 20, x + 100, y - 20)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 30, x + 38, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 38, y - 10)
			dc.DrawLine(x + 80, y - 20, x + 100, y - 20)
		adc.DrawSpline([(x + 25, y - 40), (x + 33, y - 30), (x + 35, y - 20), (x + 33, y - 10), (x + 25, y)])
		adc.DrawSpline([(x + 30, y - 40), (x + 38, y - 30), (x + 40, y - 20), (x + 38, y - 10), (x + 30, y)])
		adc.DrawSpline([(x + 30, y - 40), (x + 40, y - 40), (x + 50, y - 39), (x + 60, y - 36), (x + 70, y - 30), (x + 80, y - 20)])
		adc.DrawSpline([(x + 30, y), (x + 40, y), (x + 50, y - 1), (x + 60, y - 4), (x + 70, y - 10), (x + 80, y - 20)])
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 97 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 97 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.output_level[0] = (input_datas[0] and not input_datas[1]) or (not input_datas[0] and input_datas[1])
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_NAND(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(110, -20)]
		self.input_level = [False, False]
		self.output_level = [True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 92, y - 20, x + 110, y - 20)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			dc.DrawLine(x + 92, y - 20, x + 110, y - 20)
		dc.DrawLine(x + 30, y - 40, x + 30, y)
		adc.DrawLine(x + 30, y - 40, x + 60, y - 40)
		adc.DrawLine(x + 30, y, x + 60, y)
		adc.DrawArc(x + 60, y, x + 60, y - 40, x + 60, y - 20)
		adc.DrawCircle(x + 86, y - 20, 6)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.output_level[0] = not (input_datas[0] and input_datas[1])
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_NOR(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(110, -20)]
		self.input_level = [False, False]
		self.output_level = [True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 38, y - 30)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 38, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 92, y - 20, x + 110, y - 20)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 30, x + 38, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 38, y - 10)
			dc.DrawLine(x + 92, y - 20, x + 110, y - 20)
		adc.DrawSpline([(x + 30, y - 40), (x + 38, y - 30), (x + 40, y - 20), (x + 38, y - 10), (x + 30, y)])
		adc.DrawSpline([(x + 30, y - 40), (x + 40, y - 40), (x + 50, y - 39), (x + 60, y - 36), (x + 70, y - 30), (x + 80, y - 20)])
		adc.DrawSpline([(x + 30, y), (x + 40, y), (x + 50, y - 1), (x + 60, y - 4), (x + 70, y - 10), (x + 80, y - 20)])
		adc.DrawCircle(x + 86, y - 20, 6)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.output_level[0] = not (input_datas[0] or input_datas[1])
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_RSFF(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(100, -30), (100, -10)]
		self.input_level = [False, False]
		self.output_level = [False, True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 80, y - 30, x + 100, y - 30)
			if self.output_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 80, y - 10, x + 100, y - 10)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			dc.DrawLine(x + 80, y - 30, x + 100, y - 30)
			dc.DrawLine(x + 80, y - 10, x + 100, y - 10)
		dc.DrawRectangle(x + 30, y - 40, 50, 40)
		adc.DrawText("R", x + 35, y - 40)
		adc.DrawText("S", x + 35, y - 20)
		adc.DrawText("Q", x + 65, y - 40)
		adc.DrawText("~Q", x + 60, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.output_level[0] = True
		self.output_level[1] = False
		if input_datas[0] and not input_datas[1]:
			self.output_level[0] = True
			self.output_level[1] = False
		elif not input_datas[0] and input_datas[1]:
			self.output_level[0] = False
			self.output_level[1] = True
		elif input_datas[0] and input_datas[1]:
			self.output_level[0] = True
			self.output_level[1] = True
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_JKFF(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -50), (10, -30), (10, -10)]
		self.output_pins = [(110, -50), (110, -10)]
		self.input_level = [False, False, False]
		self.output_level = [False, True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 50, x + 30, y - 50)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			if self.input_level[2]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 90, y - 50, x + 110, y - 50)
			if self.output_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 90, y - 10, x + 110, y - 10)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 50, x + 30, y - 50)
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			dc.DrawLine(x + 90, y - 50, x + 110, y - 50)
			dc.DrawLine(x + 90, y - 10, x + 110, y - 10)
		dc.DrawRectangle(x + 30, y - 60, 60, 60)
		adc.DrawLine(x + 30, y - 35, x + 40, y - 30)
		adc.DrawLine(x + 30, y - 25, x + 40, y - 30)
		adc.DrawText("J", x + 45, y - 60)
		adc.DrawText("CK", x + 45, y - 40)
		adc.DrawText("K", x + 45, y - 20)
		adc.DrawText("Q", x + 75, y - 60)
		adc.DrawText("~Q", x + 70, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 57 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 57 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		if not self.input_level[1] and input_datas[1]: # clock (positive edge)
			if input_datas[0] and not input_datas[2]:
				self.output_level[0] = True
				self.output_level[1] = False
			elif not input_datas[0] and input_datas[2]:
				self.output_level[0] = False
				self.output_level[1] = True
			elif input_datas[0] and input_datas[2]:
				self.output_level[0] = not self.output_level[0]
				self.output_level[1] = not self.output_level[1]
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		self.input_level[2] = input_datas[2]
		return False

class Logic_DFF(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -30), (10, -10)]
		self.output_pins = [(110, -30), (110, -10)]
		self.input_level = [False, False]
		self.output_level = [False, True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 90, y - 30, x + 110, y - 30)
			if self.output_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 90, y - 10, x + 110, y - 10)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			dc.DrawLine(x + 90, y - 30, x + 110, y - 30)
			dc.DrawLine(x + 90, y - 10, x + 110, y - 10)
		dc.DrawRectangle(x + 30, y - 40, 60, 40)
		adc.DrawLine(x + 30, y - 15, x + 40, y - 10)
		adc.DrawLine(x + 30, y - 5, x + 40, y - 10)
		adc.DrawText("D", x + 45, y - 40)
		adc.DrawText("CK", x + 45, y - 20)
		adc.DrawText("Q", x + 75, y - 40)
		adc.DrawText("~Q", x + 70, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		if not self.input_level[1] and input_datas[1]: # clock (positive edge)
			self.output_level[0] = input_datas[0]
			self.output_level[1] = not input_datas[0]
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_TFF(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -20)]
		self.output_pins = [(110, -30), (110, -10)]
		self.input_level = [False]
		self.output_level = [False, True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 20, x + 30, y - 20)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 90, y - 30, x + 110, y - 30)
			if self.output_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 90, y - 10, x + 110, y - 10)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 20, x + 30, y - 20)
			dc.DrawLine(x + 90, y - 30, x + 110, y - 30)
			dc.DrawLine(x + 90, y - 10, x + 110, y - 10)
		dc.DrawRectangle(x + 30, y - 40, 60, 40)
		adc.DrawLine(x + 30, y - 25, x + 40, y - 20)
		adc.DrawLine(x + 30, y - 15, x + 40, y - 20)
		adc.DrawText("T", x + 45, y - 30)
		adc.DrawText("Q", x + 75, y - 40)
		adc.DrawText("~Q", x + 70, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 107 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 107 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		if not self.input_level[0] and input_datas[0]: # clock (positive edge)
			self.output_level[0] = not self.output_level[0]
			self.output_level[1] = not self.output_level[1]
		self.input_level[0] = input_datas[0]
		return False

class Logic_counter(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -50), (10, -30)]
		self.output_pins = [(120, -70), (120, -50), (120, -30), (120, -10)]
		self.input_level = [False, False]
		self.output_level = [False, False, False, False]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 50, x + 30, y - 50)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 100, y - 70, x + 120, y - 70)
			if self.output_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 100, y - 50, x + 120, y - 50)
			if self.output_level[2]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 100, y - 30, x + 120, y - 30)
			if self.output_level[3]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 100, y - 10, x + 120, y - 10)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 50, x + 30, y - 50)
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			dc.DrawLine(x + 100, y - 70, x + 120, y - 70)
			dc.DrawLine(x + 100, y - 50, x + 120, y - 50)
			dc.DrawLine(x + 100, y - 30, x + 120, y - 30)
			dc.DrawLine(x + 100, y - 10, x + 120, y - 10)
		dc.DrawRectangle(x + 30, y - 80, 70, 80)
		adc.DrawLine(x + 30, y - 55, x + 40, y - 50)
		adc.DrawLine(x + 30, y - 45, x + 40, y - 50)
		adc.DrawText("CK", x + 45, y - 60)
		adc.DrawText("RST", x + 45, y - 40)
		adc.DrawText("Q0", x + 80, y - 80)
		adc.DrawText("Q1", x + 80, y - 60)
		adc.DrawText("Q2", x + 80, y - 40)
		adc.DrawText("Q3", x + 80, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 117 and self.pos_y - 77 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 117 <= x2 and y1 <= self.pos_y - 77 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		if input_datas[1]:
			self.output_level[0] = False
			self.output_level[1] = False
			self.output_level[2] = False
			self.output_level[3] = False
		elif not self.input_level[0] and input_datas[0]: # clock (positive edge)
			dat = self.output_level[0] << 3 | self.output_level[1] << 2 | self.output_level[2] << 1 | self.output_level[3]
			dat += 1
			if dat == 16:
				dat = 0
			self.output_level[0] = dat >> 3 & 0x01
			self.output_level[1] = dat >> 2 & 0x01
			self.output_level[2] = dat >> 1 & 0x01
			self.output_level[3] = dat & 0x01
				
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		return False

class Logic_SW(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = []
		self.output_pins = [(70, -20)]
		self.input_level = []
		self.output_level = [False]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			tmpbrush = dc.GetBrush()
			if self.output_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 50, y - 20, x + 70, y - 20)
			dc.SetPen(tmppen)
			dc.SetBrush(wx.Brush(wx.Colour(0, 0, 0)))
			if self.output_level[0]:
				dc.DrawRectangle(x + 31, y - 28, 12, 6)
			else:
				dc.DrawRectangle(x + 17, y - 28, 12, 6)
			dc.SetBrush(tmpbrush)
		else:
			dc.DrawLine(x + 50, y - 20, x + 70, y - 20)
			dc.DrawRectangle(x + 17, y - 28, 12, 6)
		dc.DrawRectangle(x + 10, y - 40, 40, 40)
		dc.DrawRectangle(x + 15, y - 30, 30, 10)
		adc.DrawText("L", x + 15, y - 20)
		adc.DrawText("H", x + 38, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 67 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 67 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
		
	def click(self, x, y):
		self.output_level[0] = not self.output_level[0]
		return True

class Logic_7seg(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -70), (10, -50), (10, -30), (10, -10)]
		self.output_pins = []
		self.input_level = [False, False, False, False]
		self.output_level = []
		
	def set_7seg(self, x, y, adc, a, b, c, d, e, f, g):
		tmppen = adc.GetPen()
		adc.SetPen(wx.Pen(wx.Colour(128, 128, 128), 1))
		if a:
			adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
		else:
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		adc.DrawPolygon([(x + 60, y - 70), (x + 62, y - 68), (x + 88, y - 68), (x + 90, y - 70), (x + 88, y - 72), (x + 62, y - 72)])
		if b:
			adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
		else:
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		adc.DrawPolygon([(x + 90, y - 70), (x + 88, y - 68), (x + 88, y - 42), (x + 90, y - 40), (x + 92, y - 42), (x + 92, y - 68)])
		if c:
			adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
		else:
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		adc.DrawPolygon([(x + 90, y - 40), (x + 88, y - 38), (x + 88, y - 12), (x + 90, y - 10), (x + 92, y - 12), (x + 92, y - 38)])
		if d:
			adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
		else:
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		adc.DrawPolygon([(x + 60, y - 10), (x + 62, y - 8), (x + 88, y - 8), (x + 90, y - 10), (x + 88, y - 12), (x + 62, y - 12)])
		if e:
			adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
		else:
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		adc.DrawPolygon([(x + 60, y - 40), (x + 58, y - 38), (x + 58, y - 12), (x + 60, y - 10), (x + 62, y - 12), (x + 62, y - 38)])
		if f:
			adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
		else:
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		adc.DrawPolygon([(x + 60, y - 70), (x + 58, y - 68), (x + 58, y - 42), (x + 60, y - 40), (x + 62, y - 42), (x + 62, y - 68)])
		if g:
			adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
		else:
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		adc.DrawPolygon([(x + 60, y - 40), (x + 62, y - 38), (x + 88, y - 38), (x + 90, y - 40), (x + 88, y - 42), (x + 62, y - 42)])
		adc.SetPen(tmppen)
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			tmpbrush = adc.GetBrush()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 70, x + 30, y - 70)
			if self.input_level[1]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 50, x + 30, y - 50)
			if self.input_level[2]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			if self.input_level[3]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			# Draw 7seg
			if not self.input_level[0] and not self.input_level[1] and not self.input_level[2] and not self.input_level[3]:
				# 0
				self.set_7seg(x, y, adc, True, True, True, True, True, True, False)
			elif not self.input_level[0] and not self.input_level[1] and not self.input_level[2] and self.input_level[3]:
				# 1
				self.set_7seg(x, y, adc, False, True, True, False, False, False, False)
			elif not self.input_level[0] and not self.input_level[1] and self.input_level[2] and not self.input_level[3]:
				# 2
				self.set_7seg(x, y, adc, True, True, False, True, True, False, True)
			elif not self.input_level[0] and not self.input_level[1] and self.input_level[2] and self.input_level[3]:
				# 3
				self.set_7seg(x, y, adc, True, True, True, True, False, False, True)
			elif not self.input_level[0] and self.input_level[1] and not self.input_level[2] and not self.input_level[3]:
				# 4
				self.set_7seg(x, y, adc, False, True, True, False, False, True, True)
			elif not self.input_level[0] and self.input_level[1] and not self.input_level[2] and self.input_level[3]:
				# 5
				self.set_7seg(x, y, adc, True, False, True, True, False, True, True)
			elif not self.input_level[0] and self.input_level[1] and self.input_level[2] and not self.input_level[3]:
				# 6
				self.set_7seg(x, y, adc, True, False, True, True, True, True, True)
			elif not self.input_level[0] and self.input_level[1] and self.input_level[2] and self.input_level[3]:
				# 7
				self.set_7seg(x, y, adc, True, True, True, False, False, False, False)
			elif self.input_level[0] and not self.input_level[1] and not self.input_level[2] and not self.input_level[3]:
				# 8
				self.set_7seg(x, y, adc, True, True, True, True, True, True, True)
			elif self.input_level[0] and not self.input_level[1] and not self.input_level[2] and self.input_level[3]:
				# 9
				self.set_7seg(x, y, adc, True, True, True, True, False, True, True)
			elif self.input_level[0] and not self.input_level[1] and self.input_level[2] and not self.input_level[3]:
				# A
				self.set_7seg(x, y, adc, True, True, True, False, True, True, True)
			elif self.input_level[0] and not self.input_level[1] and self.input_level[2] and self.input_level[3]:
				# B
				self.set_7seg(x, y, adc, False, False, True, True, True, True, True)
			elif self.input_level[0] and self.input_level[1] and not self.input_level[2] and not self.input_level[3]:
				# C
				self.set_7seg(x, y, adc, True, False, False, True, True, True, False)
			elif self.input_level[0] and self.input_level[1] and not self.input_level[2] and self.input_level[3]:
				# D
				self.set_7seg(x, y, adc, False, True, True, True, True, False, True)
			elif self.input_level[0] and self.input_level[1] and self.input_level[2] and not self.input_level[3]:
				# E
				self.set_7seg(x, y, adc, True, False, False, True, True, True, True)
			else:
				# F
				self.set_7seg(x, y, adc, True, False, False, False, True, True, True)
			adc.SetBrush(tmpbrush)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 10, y - 70, x + 30, y - 70)
			dc.DrawLine(x + 10, y - 50, x + 30, y - 50)
			dc.DrawLine(x + 10, y - 30, x + 30, y - 30)
			dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
			adc.DrawPolygon([(x + 60, y - 70), (x + 62, y - 68), (x + 88, y - 68), (x + 90, y - 70), (x + 88, y - 72), (x + 62, y - 72)])
			adc.DrawPolygon([(x + 90, y - 70), (x + 88, y - 68), (x + 88, y - 42), (x + 90, y - 40), (x + 92, y - 42), (x + 92, y - 68)])
			adc.DrawPolygon([(x + 90, y - 40), (x + 88, y - 38), (x + 88, y - 12), (x + 90, y - 10), (x + 92, y - 12), (x + 92, y - 38)])
			adc.DrawPolygon([(x + 60, y - 10), (x + 62, y - 8), (x + 88, y - 8), (x + 90, y - 10), (x + 88, y - 12), (x + 62, y - 12)])
			adc.DrawPolygon([(x + 60, y - 40), (x + 58, y - 38), (x + 58, y - 12), (x + 60, y - 10), (x + 62, y - 12), (x + 62, y - 38)])
			adc.DrawPolygon([(x + 60, y - 70), (x + 58, y - 68), (x + 58, y - 42), (x + 60, y - 40), (x + 62, y - 42), (x + 62, y - 68)])
			adc.DrawPolygon([(x + 60, y - 40), (x + 62, y - 38), (x + 88, y - 38), (x + 90, y - 40), (x + 88, y - 42), (x + 62, y - 42)])
		dc.DrawRectangle(x + 30, y - 80, 70, 80)
		adc.DrawText("IA", x + 35, y - 80)
		adc.DrawText("IB", x + 35, y - 60)
		adc.DrawText("IC", x + 35, y - 40)
		adc.DrawText("ID", x + 35, y - 20)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 97 and self.pos_y - 77 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 97 <= x2 and y1 <= self.pos_y - 77 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		self.input_level[1] = input_datas[1]
		self.input_level[2] = input_datas[2]
		self.input_level[3] = input_datas[3]
		return False
		
class Logic_LED(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = [(10, -20)]
		self.output_pins = []
		self.input_level = [False]
		self.output_level = []
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			tmpbrush = dc.GetBrush()
			if self.input_level[0]:
				dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			else:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 10, y - 20, x + 30, y - 20)
			if self.input_level[0]:
				adc.SetBrush(wx.Brush(wx.Colour(0, 255, 0)))
			else:
				adc.SetBrush(wx.Brush(wx.Colour(0, 64, 0)))
			adc.DrawCircle(x + 50, y - 20, 8)
			adc.SetBrush(tmpbrush)
			dc.SetPen(tmppen)
		else:
			adc.DrawCircle(x + 50, y - 20, 8)
			dc.DrawLine(x + 10, y - 20, x + 30, y - 20)
		dc.DrawRectangle(x + 30, y - 40, 40, 40)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 67 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 67 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		return False

class Logic_VDD(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = []
		self.output_pins = [(20, 0)]
		self.input_level = []
		self.output_level = [True]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
			dc.DrawLine(x + 20, y - 20, x + 20, y)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 20, y - 20, x + 20, y)
		dc.DrawLine(x + 10, y - 20, x + 30, y - 20)
		dc.DrawRectangle(x + 19, y - 21, 3, 3)
		adc.DrawText("Vdd", x + 10, y - 40)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 27 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 27 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False

class Logic_GND(Logic_Component):
	def __init__(self, *args, **kwds):
		Logic_Component.__init__(self, *args, **kwds)
		self.input_pins = []
		self.output_pins = [(20, -30)]
		self.input_level = []
		self.output_level = [False]
		
	def drawComponent(self, x, y, dc, adc, running_mode):
		if running_mode:
			tmppen = dc.GetPen()
			dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
			dc.DrawLine(x + 20, y - 30, x + 20, y - 10)
			dc.SetPen(tmppen)
		else:
			dc.DrawLine(x + 20, y - 30, x + 20, y - 10)
		dc.DrawLine(x + 10, y - 10, x + 30, y - 10)
		adc.DrawLine(x + 16, y - 10, x + 12, y)
		adc.DrawLine(x + 22, y - 10, x + 18, y )
		adc.DrawLine(x + 28, y - 10, x + 24, y)
		adc.DrawText("GND", x + 35, y - 30)
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 57 and self.pos_y - 27 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 57 <= x2 and y1 <= self.pos_y - 27 and self.pos_y - 3 <= y2:
			return True
		return False

