# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import wx, const

class DrawArea(wx.Panel):
	def __init__(self, parent, *args, **kwds):
		wx.Panel.__init__(self, parent, *args, **kwds)

		self.SetBackgroundColour(wx.Colour(0, 0, 0))
		wx.EVT_PAINT(self, self.onPaint)
		wx.EVT_SCROLLWIN(self, self.onScroll)
		wx.EVT_SIZE(self, self.onSize)
		self.Bind(wx.EVT_LEAVE_WINDOW, self.onLeave)
		self.Bind(wx.EVT_ENTER_WINDOW, self.onEnter)
		self.width = 0
		self.height = 0
		self.scrollX = 0
		self.scrollY = 0
		self.cursor_x = 0
		self.cursor_y = 0
		self.parent = parent
		self.added = False
		self.preadd = False
		self.cursor_over = False
		self.mbitmap = None
		self.circuit = None
		self.redraw = True
		self.drawing_font = wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.NORMAL, False, "VL Gothic")

	def onPaint(self, evt):
		dc = wx.MemoryDC(self.mbitmap)
		scrdc = wx.PaintDC(self)
		
		if self.redraw:
			try:
				# Support antialias
				adc = wx.GCDC(dc)
			except:
				adc = dc
			
			if self.parent.running_mode:
				dc.SetBackground(wx.Brush(wx.Colour(255, 255, 255)))
			else:
				dc.SetBackground(wx.Brush(wx.Colour(0, 0, 0)))
		
			dc.Clear()
		
			if not self.parent.running_mode:
				# Draw grids
				dc.SetPen(wx.Pen(wx.Colour(32, 32, 32), 1))
				for x in range(-self.scrollX % 10, self.width, 10):
					dc.DrawLine(x, 0, x, self.height)
				for y in range(-self.scrollY % 10, self.height, 10):
					dc.DrawLine(0, y, self.width, y)
		
			# Draw net
			for c in self.circuit.components:
				if c[0] == const.component_net:
					if not c in self.circuit.selected_components:
						if self.parent.running_mode:
							for i,net in enumerate(self.circuit.net_connections):
								if (c[1], c[2]) in net:
									if self.circuit.net_levels[i] == 1:
										dc.SetPen(wx.Pen(wx.Colour(255, 0, 0), 1))
									elif self.circuit.net_levels[i] == 0:
										dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
									else:
										dc.SetPen(wx.Pen(wx.Colour(0, 0, 0), 1))
						else:
							dc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
						dc.DrawLine(c[1]-self.scrollX, c[2]-self.scrollY, c[3]-self.scrollX, c[4]-self.scrollY)
		
			# Draw component
			adc.SetFont(self.drawing_font)
			adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
			dc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
			
			if self.parent.running_mode:
				adc.SetPen(wx.Pen(wx.Colour(0, 0, 0), 1))
				adc.SetTextForeground(wx.Colour(255, 255, 255))
				adc.SetTextForeground(wx.Colour(0, 0, 0))
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 0), 1))
			else:
				adc.SetPen(wx.Pen(wx.Colour(0, 255, 0), 1))
				adc.SetTextForeground(wx.Colour(0, 255, 0))
				dc.SetPen(wx.Pen(wx.Colour(0, 255, 0), 1))
							
			for c in self.circuit.components:
				if c[0] != const.component_net:
					if not c in self.circuit.selected_components:
						c[1].drawComponent(c[1].pos_x - self.scrollX, c[1].pos_y - self.scrollY, dc, adc, self.parent.running_mode)
					
			# Draw terminal of components
			if not self.parent.running_mode:
				dc.SetPen(wx.Pen(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
				dc.SetBrush(wx.Brush(wx.Colour(255, 0, 0)))
				for c in self.circuit.components:
					if c[0] != const.component_net:
						if not c in self.circuit.selected_components:
							for p in c[1].input_pins + c[1].output_pins:
								dc.DrawRectangle(c[1].pos_x - self.scrollX + p[0] - 1, c[1].pos_y - self.scrollY + p[1] - 1, 3, 3)
				
			# Draw terminal of nets
			if self.parent.running_mode:
				dc.SetBrush(wx.Brush(wx.Colour(0, 0, 0)))
			else:
				dc.SetBrush(wx.Brush(wx.Colour(255, 0, 0)))
			dc.SetPen(wx.Pen(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
			for c in self.circuit.components:
				if c[0] == const.component_net:
					if not c in self.circuit.selected_components:
						if not self.parent.running_mode:
							dc.DrawRectangle(c[1]-self.scrollX-1, c[2]-self.scrollY-1, 3, 3)
							dc.DrawRectangle(c[3]-self.scrollX-1, c[4]-self.scrollY-1, 3, 3)
						elif not (c[1], c[2]) in self.circuit.net_no_dot:
							dc.DrawRectangle(c[1]-self.scrollX-1, c[2]-self.scrollY-1, 3, 3)
						elif not (c[3], c[4]) in self.circuit.net_no_dot:
							dc.DrawRectangle(c[3]-self.scrollX-1, c[4]-self.scrollY-1, 3, 3)
		
			# Draw selected components
			if not self.parent.running_mode:
				if not self.parent.component_dragged:
					adc.SetTextForeground(wx.Colour(255, 255, 255))
					adc.SetPen(wx.Pen(wx.Colour(255, 255, 255), 1))
					dc.SetPen(wx.Pen(wx.Colour(255, 255, 255), 1))
					for c in self.circuit.selected_components:
						if c[0] == const.component_net:
							dc.DrawLine(c[1]-self.scrollX, c[2]-self.scrollY, c[3]-self.scrollX, c[4]-self.scrollY)
				
					adc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
					dc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
					for c in self.circuit.selected_components:
						if c[0] != const.component_net:
							c[1].drawComponent(c[1].pos_x - self.scrollX, c[1].pos_y - self.scrollY, dc, adc, self.parent.running_mode)
							
					dc.SetPen(wx.Pen(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
					dc.SetBrush(wx.Brush(wx.Colour(255, 255, 255)))
					for c in self.circuit.selected_components:
						if c[0] != const.component_net:
							for p in c[1].input_pins + c[1].output_pins:
								dc.DrawRectangle(c[1].pos_x - self.scrollX + p[0] - 1, c[1].pos_y - self.scrollY + p[1] - 1, 3, 3)
								
					for c in self.circuit.selected_components:
						if c[0] == const.component_net:
							dc.DrawRectangle(c[1]-self.scrollX-1, c[2]-self.scrollY-1, 3, 3)
							dc.DrawRectangle(c[3]-self.scrollX-1, c[4]-self.scrollY-1, 3, 3)
							
		# Draw buffered image
		scrdc.Blit(0, 0, self.width, self.height, dc, 0, 0)
		dc.SelectObject(wx.NullBitmap)
		
		# Draw unbuffered graphics
		try:
			# Support antialias
			scradc = wx.GCDC(scrdc)
		except:
			scradc = scrdc
			
		scradc.SetFont(self.drawing_font)
		scrdc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
		scradc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
				
		if self.parent.rect_select_enabled:
			scrgc = wx.GraphicsContext.Create(scrdc)
			scrgc.PushState()
			scrgc.SetPen(wx.Pen(wx.Colour(255, 192, 0), 1))
			scrgc.SetBrush(wx.Brush(wx.Colour(255, 192, 0, 64)))
			scrgc.DrawRectangle(self.parent.select_start_x - self.scrollX, self.parent.select_start_y - self.scrollY, self.cursor_smooth_x - self.parent.select_start_x, self.cursor_smooth_y- self.parent.select_start_y)
			scrgc.PopState()
		
		# Draw choosed component
		if not self.added and self.cursor_over and self.parent.comp_window.checked_button != const.component_none:
			if self.parent.comp_window.checked_button == const.component_net:
				# Draw cursor
				scradc.SetPen(wx.Pen(wx.Colour(255, 255, 255), 1))
				scradc.DrawCircle(self.cursor_x-self.scrollX, self.cursor_y-self.scrollY, 3)
				if self.netstarted:
					# Draw net
					scrdc.SetPen(wx.Pen(wx.Colour(0, 0, 255), 1))
					if self.netstart_x != self.cursor_x:
						scrdc.DrawLine(self.netstart_x-self.scrollX, self.netstart_y-self.scrollY, self.cursor_x-self.scrollX, self.netstart_y-self.scrollY)
					if self.netstart_y != self.cursor_y:
						scrdc.DrawLine(self.cursor_x-self.scrollX, self.netstart_y-self.scrollY, self.cursor_x-self.scrollX, self.cursor_y-self.scrollY)
						
					scrdc.SetPen(wx.Pen(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
					scrdc.SetBrush(wx.Brush(wx.Colour(255, 0, 255)))
					scrdc.DrawRectangle(self.netstart_x-self.scrollX-1, self.netstart_y-self.scrollY-1, 3, 3)
					scrdc.DrawRectangle(self.cursor_x-self.scrollX-1, self.netstart_y-self.scrollY-1, 3, 3)
						
			else:
				# Draw picked component
				if self.preadd:
					scradc.SetPen(wx.Pen(wx.Colour(255, 192, 0), 1))
					scradc.SetTextForeground(wx.Colour(255, 192, 0))
					scrdc.SetPen(wx.Pen(wx.Colour(255, 192, 0), 1))
				else:
					scradc.SetPen(wx.Pen(wx.Colour(255, 128, 0), 1))
					scradc.SetTextForeground(wx.Colour(255, 128, 0))
					scrdc.SetPen(wx.Pen(wx.Colour(255, 128, 0), 1))
					
				logicand = self.pushed_component
				logicand.drawComponent(self.cursor_x-self.scrollX, self.cursor_y-self.scrollY, scrdc, scradc, self.parent.running_mode)
		
		# Draw selected components
		if not self.parent.running_mode:
			if self.parent.component_dragged:
				scradc.SetTextForeground(wx.Colour(255, 255, 255))
				scradc.SetPen(wx.Pen(wx.Colour(255, 255, 255), 1))
				scrdc.SetPen(wx.Pen(wx.Colour(255, 255, 255), 1))
				for c in self.circuit.selected_components:
					if c[0] == const.component_net:
						scrdc.DrawLine(c[1]-self.scrollX, c[2]-self.scrollY, c[3]-self.scrollX, c[4]-self.scrollY)
			
				scradc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
				scrdc.SetBrush(wx.Brush(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
				for c in self.circuit.selected_components:
					if c[0] != const.component_net:
						c[1].drawComponent(c[1].pos_x - self.scrollX, c[1].pos_y - self.scrollY, scrdc, scradc, self.parent.running_mode)
						
				scrdc.SetPen(wx.Pen(wx.Colour(0, 0, 0), style=wx.TRANSPARENT))
				scrdc.SetBrush(wx.Brush(wx.Colour(255, 255, 255)))
				for c in self.circuit.selected_components:
					if c[0] != const.component_net:
						for p in c[1].input_pins + c[1].output_pins:
							scrdc.DrawRectangle(c[1].pos_x - self.scrollX + p[0] - 1, c[1].pos_y - self.scrollY + p[1] - 1, 3, 3)
							
				for c in self.circuit.selected_components:
					if c[0] == const.component_net:
						scrdc.DrawRectangle(c[1]-self.scrollX-1, c[2]-self.scrollY-1, 3, 3)
						scrdc.DrawRectangle(c[3]-self.scrollX-1, c[4]-self.scrollY-1, 3, 3)
						
		# Highlight component
		if self.parent.nearest_component is not None and not self.parent.nearest_component in self.circuit.selected_components and not self.parent.running_mode:
			c = self.parent.nearest_component
			if c[0] == const.component_net:
				scrdc.SetPen(wx.Pen(wx.Colour(128, 128, 255), 1))
				scrdc.DrawLine(c[1]-self.scrollX, c[2]-self.scrollY, c[3]-self.scrollX, c[4]-self.scrollY)
			else:
				scradc.SetPen(wx.Pen(wx.Colour(128, 255, 128), 1))
				scradc.SetTextForeground(wx.Colour(128, 255, 128))
				scrdc.SetPen(wx.Pen(wx.Colour(128, 255, 128), 1))
				c[1].drawComponent(c[1].pos_x - self.scrollX, c[1].pos_y - self.scrollY, scrdc, scradc, self.parent.running_mode)
				
		self.redraw = False
		self.added = False
		
	def onScroll(self, evt):
		self.scrollX = self.GetScrollPos(wx.HORIZONTAL)
		self.scrollY = self.GetScrollPos(wx.VERTICAL)
		self.redraw = True
		self.Refresh()
	
	def onSize(self, evt):
		old_width, old_height = self.width, self.height
		self.width, self.height = self.ClientSize
		self.SetScrollbar(wx.VERTICAL, 0, self.height, 1080);
		self.SetScrollbar(wx.HORIZONTAL, 0, self.width, 1920);
		if old_width < self.width or old_height < self.height:
			self.mbitmap = wx.EmptyBitmap(self.width, self.height)
			self.redraw = True
			self.Refresh()
		
	def onLeave(self, event):
		self.cursor_over = False
		if self.parent.comp_window.checked_button != const.component_none:
			self.Refresh()
	
	def onEnter(self, event):
		self.cursor_over = True
		self.parent.SetStatusText(self.parent.statusbar_text)

