# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import wx, config, const, copy, os, gettext, sys
from DrawArea import DrawArea
from ComponentWindow import ComponentWindow
from CircuitManager import *

gettext.textdomain("wxlogic")

def _(x):
	return gettext.gettext(x).decode("utf-8")

class MainFrame(wx.Frame):
	def __init__(self, *args, **kwds):
		kwds["style"] = wx.DEFAULT_FRAME_STYLE
		wx.Frame.__init__(self, *args, **kwds)
		
		self.running_mode = False
		
		self.circuit = CircuitManager(self)
		self.nearest_component = None
		self.preselected_component = None
		self.drag_enabled = False
		self.component_dragged = False
		self.rect_select_enabled = False
		self.mouse_down = False
		self.drag_delta_x = 0
		self.drag_delta_y = 0
		self.select_start_x = 0
		self.select_start_y = 0
		self.statusbar_text = ""
		
		self.create_window()
		
		self.SetMenuBar(self.menubar)
		
		if len(sys.argv) >= 2:
			self.circuit.open_file(sys.argv[1])
			self.drawarea.redraw = True
			self.drawarea.Refresh()
	
	def set_status(self, text):
		self.statusbar_text = text
		self.SetStatusText(text)
	
	def create_window(self):
		self.SetTitle(u"%s - wxLogic" % const.text_notitle)
		self.SetIcon(wx.Icon(config.DATADIR+"/images/wxlogic-small.ico", wx.BITMAP_TYPE_ICO))
		
		# Menu bar
		self.menubar = wx.MenuBar()

		menu_file = wx.Menu()
		menu_new = wx.MenuItem(menu_file, wx.ID_NEW, u"%s\tCtrl+N" % _("&New"))
		menu_open = wx.MenuItem(menu_file, wx.ID_OPEN, u"%s\tCtrl+O" % _("&Open..."))
		menu_save = wx.MenuItem(menu_file, wx.ID_SAVE, u"%s\tCtrl+S" % _("&Save"))
		menu_saveas = wx.MenuItem(menu_file, wx.ID_SAVEAS, u"%s\tShift+Ctrl+S" % _("Save &As..."))
		menu_quit = wx.MenuItem(menu_file, wx.ID_EXIT, u"%s\tCtrl+Q" % _("&Quit"))
		menu_file.AppendItem(menu_new)
		menu_file.AppendItem(menu_open)
		menu_file.AppendSeparator()
		menu_file.AppendItem(menu_save)
		menu_file.AppendItem(menu_saveas)
		menu_file.AppendSeparator()
		menu_file.AppendItem(menu_quit)
		self.menubar.Append(menu_file, _("&File"))

		menu_edit = wx.Menu()
		self.menu_undo = wx.MenuItem(menu_edit, wx.ID_UNDO, u"%s\tCtrl+Z" % _("&Undo"))
		self.menu_redo = wx.MenuItem(menu_edit, wx.ID_REDO, u"%s\tShift+Ctrl+Z" % _("&Redo"))
		self.menu_delete = wx.MenuItem(menu_edit, wx.ID_DELETE, u"%s\tDelete" % _("&Delete"))
		menu_edit.AppendItem(self.menu_undo)
		menu_edit.AppendItem(self.menu_redo)
		menu_edit.AppendSeparator()
		menu_edit.AppendItem(self.menu_delete)
		self.menu_delete.Enable(False)
		self.menu_undo.Enable(False)
		self.menu_redo.Enable(False)
		self.menubar.Append(menu_edit, _("&Edit"))

		menu_add = wx.Menu()
		self.menu_components = wx.MenuItem(menu_add, 1, u"%s\tCtrl+A" % _("&Components"), kind=wx.ITEM_CHECK)
		self.menu_net = wx.MenuItem(menu_add, 2, u"%s\tCtrl+Space" % _("&Net"), kind=wx.ITEM_CHECK)
		menu_add.AppendItem(self.menu_components)
		menu_add.AppendItem(self.menu_net)
		self.menubar.Append(menu_add, _("&Add"))

		menu_simulate = wx.Menu()
		self.menu_run = wx.MenuItem(menu_simulate, 3, u"%s\tF5" % _("&Run"), kind=wx.ITEM_CHECK)
		menu_simulate.AppendItem(self.menu_run)
		self.menubar.Append(menu_simulate, _("&Simulate"))
		
		# Tool bar
		self.toolbar = self.CreateToolBar()
		
		icon_new = wx.ArtProvider.GetBitmap(wx.ART_NEW, client=wx.ART_TOOLBAR)
		icon_open = wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, client=wx.ART_TOOLBAR)
		icon_save = wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE, client=wx.ART_TOOLBAR)
		icon_undo = wx.ArtProvider.GetBitmap(wx.ART_UNDO, client=wx.ART_TOOLBAR)
		icon_redo = wx.ArtProvider.GetBitmap(wx.ART_REDO, client=wx.ART_TOOLBAR)
		if icon_new is None:
			icon_new = wx.Bitmap(config.DATADIR+"/images/document-new.png")
		if icon_open is None:
			icon_open = wx.Bitmap(config.DATADIR+"/images/document-open.png")
		if icon_save is None:
			icon_save = wx.Bitmap(config.DATADIR+"/images/document-save.png")
		if icon_undo is None:
			icon_undo = wx.Bitmap(config.DATADIR+"/images/edit-undo.png")
		if icon_redo is None:
			icon_redo = wx.Bitmap(config.DATADIR+"/images/edit-redo.png")
		
		self.toolbar.AddLabelTool(wx.ID_NEW, "", icon_new, shortHelp=_("Create new circuit"), longHelp=_("Close this circuit and create a new one."))
		self.toolbar.AddLabelTool(wx.ID_OPEN, "", icon_open, shortHelp=_("Open circuit"), longHelp=_("Close this circuit and open the other one."))
		self.toolbar.AddLabelTool(wx.ID_SAVE, "", icon_save, shortHelp=_("Save circuit"), longHelp=_("Save this circuit."))
		self.toolbar.AddSeparator()
		self.toolbar.AddLabelTool(wx.ID_UNDO, "", icon_undo, shortHelp=_("Undo"), longHelp=_("Undo the previous action."))
		self.toolbar.AddLabelTool(wx.ID_REDO, "", icon_redo, shortHelp=_("Redo"), longHelp=_("Redo the action that you have canceled."))
		self.toolbar.AddSeparator()
		self.btn_add_component = self.toolbar.AddCheckLabelTool(1, "", wx.Bitmap(config.DATADIR+"/images/add-component.png"), shortHelp=_("Add component"), longHelp=_("Add components to this circuit."))
		self.btn_add_net = self.toolbar.AddCheckLabelTool(2, "", wx.Bitmap(config.DATADIR+"/images/add-net.png"), shortHelp=_("Add nets"), longHelp=_("Add nets to this circuit."))
		self.toolbar.AddSeparator()
		self.btn_play = self.toolbar.AddCheckLabelTool(3, "", wx.Bitmap(config.DATADIR+"/images/media-playback-start.png"), shortHelp=_("Run"), longHelp=_("Run and simulate this circuit."))
		self.toolbar.EnableTool(wx.ID_UNDO, False)
		self.toolbar.EnableTool(wx.ID_REDO, False)
		self.toolbar.Realize()
		
		# Component window
		self.comp_window = ComponentWindow(self)
		
		# Draw area
		self.drawarea = DrawArea(self)
		self.drawarea.circuit = self.circuit
		self.drawarea.pushed_component = const.comp_dict[const.component_none]
		self.drawarea.netstarted = False
		
		# Status bar
		self.CreateStatusBar()
		self.SetStatusText("")
		
		# Bind events
		self.Bind(wx.EVT_MENU, self.on_menu_new_click, menu_new)
		self.Bind(wx.EVT_MENU, self.on_menu_open_click, menu_open)
		self.Bind(wx.EVT_MENU, self.on_menu_save_click, menu_save)
		self.Bind(wx.EVT_MENU, self.on_menu_saveas_click, menu_saveas)
		self.Bind(wx.EVT_MENU, self.on_menu_quit_click, menu_quit)
		self.Bind(wx.EVT_MENU, self.on_menu_undo_click, self.menu_undo)
		self.Bind(wx.EVT_MENU, self.on_menu_redo_click, self.menu_redo)
		self.Bind(wx.EVT_MENU, self.on_menu_delete_click, self.menu_delete)
		self.Bind(wx.EVT_TOOL, self.on_btn_add_component_click, self.btn_add_component)
		self.Bind(wx.EVT_TOOL, self.on_btn_add_net_click, self.btn_add_net)
		self.Bind(wx.EVT_TOOL, self.on_btn_play_click, self.btn_play)
		self.Bind(wx.EVT_CLOSE, self.on_close, self)
		self.comp_window.Bind(wx.EVT_CLOSE, self.on_comp_window_close)
		self.drawarea.Bind(wx.EVT_LEFT_DOWN, self.on_drawarea_leftdown)
		self.drawarea.Bind(wx.EVT_LEFT_UP, self.on_drawarea_leftup)
		self.drawarea.Bind(wx.EVT_RIGHT_UP, self.on_drawarea_rightup)
		self.drawarea.Bind(wx.EVT_MOTION, self.on_drawarea_motion)
	
	def reset_frame(self):
		self.comp_window.uncheck_all_buttons()
		self.SetStatusText("")
		self.running_mode = False
		self.toolbar.ToggleTool(3, False)
		self.comp_window.checked_button = const.component_none
		self.drawarea.pushed_component = const.comp_dict[const.component_none]
		self.menu_delete.Enable(False)
		self.menu_undo.Enable(False)
		self.menu_redo.Enable(False)
		self.toolbar.EnableTool(wx.ID_UNDO, False)
		self.toolbar.EnableTool(wx.ID_REDO, False)
		self.toolbar.EnableTool(1, True)
		self.toolbar.EnableTool(2, True)
		self.drawarea.SetBackgroundColour(wx.Colour(0, 0, 0))
		
	def on_menu_new_click(self, event):
		if self.check_modified():
			return
			
		self.SetTitle(u"%s - wxLogic" % const.text_notitle)
		self.reset_frame()
		self.circuit.reset_circuit()
		self.drawarea.redraw = True
		self.drawarea.Refresh()
		
	def on_menu_open_click(self, event):
		if self.check_modified():
			return
		while True:
			dialog = wx.FileDialog(self, message=_("Open file"), wildcard="%s|*.wxl|%s|*.*" % (_("wxLogic files (*.wxl)"), _("All files (*.*)")), style=wx.OPEN)
			dialog.MakeModal(True)
			if dialog.ShowModal() == wx.ID_OK:
				filepath = dialog.GetPath()
			else:
				break
				
			if not self.circuit.open_file(filepath):
				self.reset_frame()
				self.drawarea.redraw = True
				self.drawarea.Refresh()
				break
				
		dialog.MakeModal(False)
			
	def overwrite_save(self):
		if self.circuit.filepath == "":
			return self.rename_save()
		else:
			if self.circuit.save_file(self.circuit.filepath):
				return self.rename_save()
			return False
			
	def rename_save(self):
		while True:
			dialog = wx.FileDialog(self, message=_("Save file"), wildcard="%s|*.wxl|%s|*.*" % (_("wxLogic files (*.wxl)"), _("All files (*.*)")), style=wx.SAVE)
			dialog.MakeModal(True)
			if dialog.ShowModal() == wx.ID_OK:
				filepath = dialog.GetPath()
				filter_id = dialog.GetFilterIndex()
				if filter_id == 0:
					if not "." in os.path.basename(filepath):
						filepath += ".wxl"
						
				if os.path.exists(filepath):
					dialog = wx.MessageDialog(self, _("The file already exist. Overwrite it?"), _("Overwriting"), wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)
					if dialog.ShowModal() == wx.ID_NO:
						continue
			else:
				dialog.MakeModal(False)
				return True
				
			if not self.circuit.save_file(filepath):
				dialog.MakeModal(False)
				return False
	
	def on_menu_save_click(self, event):
		self.overwrite_save()

	def on_menu_saveas_click(self, event):
		self.rename_save()
		
	def on_menu_quit_click(self, event):
		self.Close()
		
	def on_close(self, event):
		if not self.check_modified():
			self.Destroy()
		
	def check_modified(self):
		if self.circuit.need_save:
			dialog = wx.MessageDialog(self, _("The schematics was modifed. Save the changes before closing?"), _("Confirmation"), wx.YES_NO | wx.CANCEL | wx.YES_DEFAULT | wx.ICON_QUESTION)
			retval = dialog.ShowModal()
			if retval == wx.ID_YES:
				return self.overwrite_save()
			elif retval == wx.ID_NO:
				return False
			else:
				return True
		return False
		
	def on_menu_undo_click(self, event):
		self.circuit.undo()
		self.menu_delete.Enable(False)
		if self.circuit.action_count == 0:
			self.toolbar.EnableTool(wx.ID_UNDO, False)
			self.menu_undo.Enable(False)
		self.toolbar.EnableTool(wx.ID_REDO, True)
		self.menu_redo.Enable(True)
		self.drawarea.redraw = True
		self.drawarea.Refresh()
		
	def on_menu_redo_click(self, event):
		self.circuit.redo()
		self.menu_delete.Enable(False)
		self.toolbar.EnableTool(wx.ID_UNDO, True)
		self.menu_undo.Enable(True)
		if self.circuit.action_count == len(self.circuit.components_history) - 1:
			self.toolbar.EnableTool(wx.ID_REDO, False)
			self.menu_redo.Enable(False)
		self.drawarea.redraw = True
		self.drawarea.Refresh()
	
	def on_menu_delete_click(self, event):
		self.circuit.remove_selected_component()
		self.menu_delete.Enable(False)
		self.circuit.push_history()
		self.toolbar.EnableTool(wx.ID_UNDO, True)
		self.toolbar.EnableTool(wx.ID_REDO, False)
		self.menu_undo.Enable(True)
		self.menu_redo.Enable(False)
		self.drawarea.redraw = True
		self.drawarea.Refresh()
		
	def on_btn_add_component_click(self, event):
		if event.IsChecked():
			self.toolbar.ToggleTool(1, True)
			self.menu_components.Check(True)
			self.comp_window.Show()
		else:
			self.toolbar.ToggleTool(1, False)
			self.menu_components.Check(False)
			self.comp_window.Hide()
			self.comp_window.uncheck_all_buttons()
			self.comp_window.checked_button = const.component_none
			self.drawarea.pushed_component = const.comp_dict[const.component_none]
			self.drawarea.Refresh()
	
	def on_btn_add_net_click(self, event):
		if event.IsChecked():
			self.comp_window.uncheck_all_buttons()
			self.toolbar.ToggleTool(2, True)
			self.menu_net.Check(True)
			self.comp_window.checked_button = const.component_net
			self.drawarea.pushed_component = const.comp_dict[const.component_net]
		else:
			self.toolbar.ToggleTool(2, False)
			self.menu_net.Check(False)
			self.comp_window.checked_button = const.component_none
			self.drawarea.pushed_component = const.comp_dict[const.component_none]
		self.drawarea.Refresh()
	
	def on_btn_play_click(self, event):
		if self.running_mode:
			self.running_mode = False
			if self.circuit.action_count > 0:
				self.toolbar.EnableTool(wx.ID_UNDO, True)
				self.menu_undo.Enable(True)
			if self.circuit.action_count < len(self.circuit.components_history) - 1:
				self.toolbar.EnableTool(wx.ID_REDO, True)
				self.menu_redo.Enable(True)
			self.toolbar.ToggleTool(3, False)
			self.menu_run.Check(False)
			self.toolbar.EnableTool(1, True)
			self.toolbar.EnableTool(2, True)
			self.drawarea.SetBackgroundColour(wx.Colour(0, 0, 0))
			self.set_status("")
		else:
			self.running_mode = True
			self.circuit.selected_components = []
			self.menu_delete.Enable(False)
			self.menu_undo.Enable(False)
			self.menu_redo.Enable(False)
			self.comp_window.Hide()
			self.comp_window.checked_button = const.component_none
			self.drawarea.pushed_component = const.comp_dict[const.component_none]
			self.toolbar.ToggleTool(1, False)
			self.toolbar.ToggleTool(2, False)
			self.toolbar.ToggleTool(3, True)
			self.menu_run.Check(True)
			self.toolbar.EnableTool(wx.ID_UNDO, False)
			self.toolbar.EnableTool(wx.ID_REDO, False)
			self.toolbar.EnableTool(1, False)
			self.toolbar.EnableTool(2, False)
			self.drawarea.SetBackgroundColour(wx.Colour(255, 255, 255))
			self.circuit.analyze_connections()
			self.circuit.analyze_logic()
		
		self.drawarea.redraw = True
		self.drawarea.Refresh()

	def on_comp_window_close(self, event):
		self.toolbar.ToggleTool(1, False)
		self.comp_window.uncheck_all_buttons()
		self.comp_window.checked_button = const.component_none
		self.drawarea.pushed_component = const.comp_dict[const.component_none]
		self.comp_window.Hide()
		self.drawarea.Refresh()

	def on_drawarea_leftdown(self, evt):
		if not self.running_mode:
			if self.comp_window.checked_button == const.component_none:
				# Check selected area
				self.rect_select_enabled = True
				for c in self.circuit.components:
					if c[0] == const.component_net:
						if (c[1] - 3 <= self.drawarea.cursor_smooth_x <= c[3] + 3 or c[3] - 3 <= self.drawarea.cursor_smooth_x <= c[1] + 3) and (c[2] - 3 <= self.drawarea.cursor_smooth_y <= c[4] + 3 or c[4] - 3 <= self.drawarea.cursor_smooth_y <= c[2] + 3):
							if ((c[4] - c[2]) * (self.drawarea.cursor_smooth_x - c[1]) + (c[3] - c[1]) * (self.drawarea.cursor_smooth_y - c[2])) ** 2 / ((c[3] - c[1]) ** 2 + (c[4] - c[2]) ** 2) <= 9:
								if c in self.circuit.selected_components:
									self.drag_delta_x = 0
									self.drag_delta_y = 0
									self.drag_enabled = True
								
								self.preselected_component = c
								self.rect_select_enabled = False
								break
					else:
						if c[1].isMouseOvered(self.drawarea.cursor_smooth_x, self.drawarea.cursor_smooth_y):
							if c in self.circuit.selected_components:
								self.drag_delta_x = 0
								self.drag_delta_y = 0
								self.drag_enabled = True
							
							self.preselected_component = c
							self.rect_select_enabled = False
							break
							
				if self.rect_select_enabled:
					self.select_start_x = self.drawarea.cursor_smooth_x
					self.select_start_y = self.drawarea.cursor_smooth_y
				
			else:
				self.drawarea.preadd = True
		else:
			for c in self.circuit.components:
				if c[0] != const.component_net:
					c[1].mouse_down(evt.X + self.drawarea.scrollX, evt.Y + self.drawarea.scrollY)
		self.mouse_down = True

	def on_drawarea_leftup(self, evt):
		if not self.running_mode:
			self.drag_enabled = False
			if self.component_dragged:
				self.component_dragged = False
				tmp_components = self.circuit.components[:]
				for c in tmp_components:
					if c[0] == const.component_net:
						self.circuit.split_nets(c[1], c[2])
						self.circuit.split_nets(c[3], c[4])
					else:
						for p in c[1].input_pins + c[1].output_pins:
							self.circuit.split_nets(c[1].pos_x + p[0], c[1].pos_y + p[1])
				
				tmp_components = self.circuit.components[:]
				for c in tmp_components:
					if c[0] == const.component_net:
						self.circuit.connect_nets(c[1], c[2])
						self.circuit.connect_nets(c[3], c[4])
							
				self.preselected_component = None
				self.circuit.push_history()
				self.toolbar.EnableTool(wx.ID_UNDO, True)
				self.toolbar.EnableTool(wx.ID_REDO, False)
				self.menu_undo.Enable(True)
				self.menu_redo.Enable(False)
			
			elif self.comp_window.checked_button == const.component_none:
				# Select component
				selected = False
				if self.preselected_component is not None:
					c = self.preselected_component
					if c[0] == const.component_net:
						if (c[1] - 3 <= self.drawarea.cursor_smooth_x <= c[3] + 3 or c[3] - 3 <= self.drawarea.cursor_smooth_x <= c[1] + 3) and (c[2] - 3 <= self.drawarea.cursor_smooth_y <= c[4] + 3 or c[4] - 3 <= self.drawarea.cursor_smooth_y <= c[2] + 3):
							if ((c[4] - c[2]) * (self.drawarea.cursor_smooth_x - c[1]) + (c[3] - c[1]) * (self.drawarea.cursor_smooth_y - c[2])) ** 2 / ((c[3] - c[1]) ** 2 + (c[4] - c[2]) ** 2) <= 9:
								if evt.ControlDown():
									if not c in self.circuit.selected_components:
										self.circuit.selected_components.append(c)
									else:
										self.circuit.selected_components.remove(c)
								else:
									self.circuit.selected_components = [c]
								selected = True
								
					elif c[1].isMouseOvered(self.drawarea.cursor_smooth_x, self.drawarea.cursor_smooth_y):
						if evt.ControlDown():
							if not c in self.circuit.selected_components:
								self.circuit.selected_components.append(c)
							else:
								self.circuit.selected_components.remove(c)
						else:
							self.circuit.selected_components = [c]
						selected = True
							
					self.preselected_component = None
				
				elif self.rect_select_enabled:
					self.rect_select_enabled = False
					rectselected_components = []
					if self.select_start_x < self.drawarea.cursor_smooth_x:
						rect_left = self.select_start_x
						rect_right = self.drawarea.cursor_smooth_x
					else:
						rect_left = self.drawarea.cursor_smooth_x
						rect_right = self.select_start_x
					if self.select_start_y < self.drawarea.cursor_smooth_y:
						rect_top = self.select_start_y
						rect_bottom = self.drawarea.cursor_smooth_y
					else:
						rect_top = self.drawarea.cursor_smooth_y
						rect_bottom = self.select_start_y
						
					for c in self.circuit.components:
						if c[0] == const.component_net:
							if rect_left <= c[1] <= rect_right and rect_left <= c[3] <= rect_right and rect_top <= c[2] <= rect_bottom and rect_top <= c[4] <= rect_bottom:
								rectselected_components.append(c)
								selected = True
						elif c[1].isInRectangle(rect_left, rect_top, rect_right, rect_bottom):
								rectselected_components.append(c)
								selected = True
								
					if evt.ControlDown():
						for c in rectselected_components:
							if not c in self.circuit.selected_components:
								self.circuit.selected_components.append(c)
								
					else:
						self.circuit.selected_components = rectselected_components
				
				if not selected and not evt.ControlDown():
					self.circuit.selected_components = []
					
				if len(self.circuit.selected_components) == 0:
					self.menu_delete.Enable(False)
				else:
					self.menu_delete.Enable(True)
				
			elif self.comp_window.checked_button == const.component_net:
				if self.drawarea.netstarted:
					# Add net
					if self.drawarea.netstart_x != self.drawarea.cursor_x:
						component_data = [const.component_net, self.drawarea.netstart_x, self.drawarea.netstart_y, self.drawarea.cursor_x, self.drawarea.netstart_y]
						component_data2 = [const.component_net, self.drawarea.cursor_x, self.drawarea.netstart_y, self.drawarea.netstart_x, self.drawarea.netstart_y]
						if not component_data in self.circuit.components and not component_data2 in self.circuit.components:
							self.circuit.components.append(component_data)
							
					if self.drawarea.netstart_y != self.drawarea.cursor_y:
						component_data = [const.component_net, self.drawarea.cursor_x, self.drawarea.netstart_y, self.drawarea.cursor_x, self.drawarea.cursor_y]
						component_data2 = [const.component_net, self.drawarea.cursor_x, self.drawarea.cursor_y, self.drawarea.cursor_x, self.drawarea.netstart_y]
						if not component_data in self.circuit.components and not component_data2 in self.circuit.components:
							self.circuit.components.append(component_data)
					
					tmp_components = self.circuit.components[:]
					for c in tmp_components:
						if c[0] == const.component_net:
							self.circuit.split_nets(c[1], c[2])
							self.circuit.split_nets(c[3], c[4])
						else:
							for p in c[1].input_pins + c[1].output_pins:
								self.circuit.split_nets(c[1].pos_x + p[0], c[1].pos_y + p[1])
								
					tmp_components = self.circuit.components[:]
					for c in tmp_components:
						if c[0] == const.component_net:
							self.circuit.connect_nets(c[1], c[2])
							self.circuit.connect_nets(c[3], c[4])
							
					self.circuit.push_history()
					self.toolbar.EnableTool(wx.ID_UNDO, True)
					self.toolbar.EnableTool(wx.ID_REDO, False)
					self.menu_undo.Enable(True)
					self.menu_redo.Enable(False)
								
				# Begin to create net
				self.drawarea.netstart_x = self.drawarea.cursor_x
				self.drawarea.netstart_y = self.drawarea.cursor_y
				self.drawarea.netstarted = True
				
			else:
				# Add component
				component_data = [self.comp_window.checked_button, copy.deepcopy(const.comp_dict[self.comp_window.checked_button])]
				component_data[1].pos_x = self.drawarea.cursor_x
				component_data[1].pos_y = self.drawarea.cursor_y
				for p in component_data[1].input_pins + component_data[1].output_pins:
					self.circuit.split_nets(p[0] + self.drawarea.cursor_x, p[1] + self.drawarea.cursor_y)
					
				self.circuit.components.append(component_data)
				self.circuit.push_history()
				self.toolbar.EnableTool(wx.ID_UNDO, True)
				self.toolbar.EnableTool(wx.ID_REDO, False)
				self.menu_undo.Enable(True)
				self.menu_redo.Enable(False)
					
			self.drawarea.preadd = False
			self.drawarea.added = True
			self.mouse_down = False
			self.drawarea.redraw = True
			self.drawarea.Refresh()
			
		else:
			for c in self.circuit.components:
				if c[0] != const.component_net:
					if c[1].mouse_up(evt.X + self.drawarea.scrollX, evt.Y + self.drawarea.scrollY):
						self.circuit.analyze_logic()
						self.drawarea.redraw = True
						self.drawarea.Refresh()
						break
			

	def on_drawarea_rightup(self, evt):
		if not self.running_mode:
			if not self.drawarea.netstarted:
				self.comp_window.uncheck_all_buttons()
				self.toolbar.ToggleTool(2, False)
				self.comp_window.checked_button = const.component_none
				self.drawarea.pushed_component = const.comp_dict[const.component_none]
			else:
				# Finish creating net
				self.drawarea.netstarted = False
				
		self.drawarea.Refresh()

	def on_drawarea_motion(self, evt):
		self.px = evt.X
		self.py = evt.Y
		
		oldcursor_x = self.drawarea.cursor_x
		oldcursor_y = self.drawarea.cursor_y
		
		tmp_x = self.px + self.drawarea.scrollX
		tmp_y = self.py + self.drawarea.scrollY
		
		if self.comp_window.checked_button != const.component_none:
			# snap cursor to terminals
			min_dist = 225
			if self.comp_window.checked_button == const.component_net and not evt.ControlDown():
				for c in self.circuit.components:
					if c[0] == const.component_net:
						if min_dist > (tmp_x - c[1]) ** 2 + (tmp_y - c[2]) ** 2:
							min_dist = (tmp_x - c[1]) ** 2 + (tmp_y - c[2]) ** 2
							self.drawarea.cursor_x = c[1]
							self.drawarea.cursor_y = c[2]
						if min_dist > (tmp_x - c[3]) ** 2 + (tmp_y - c[4]) ** 2:
							min_dist = (tmp_x - c[3]) ** 2 + (tmp_y - c[4]) ** 2
							self.drawarea.cursor_x = c[3]
							self.drawarea.cursor_y = c[4]
						if c[1] == c[3]:
							if c[2] < tmp_y < c[4] or c[4] < tmp_y < c[2]:
								if min_dist > (tmp_x - c[1]) ** 2:
									min_dist = (tmp_x - c[1]) ** 2
									self.drawarea.cursor_x = c[1] - c[1] % 10
									self.drawarea.cursor_y = tmp_y - tmp_y % 10
						if c[2] == c[4]:
							if c[1] < tmp_x < c[3] or c[3] < tmp_x < c[1]:
								if  min_dist > (tmp_y - c[2]) ** 2:
									min_dist = (tmp_y - c[2]) ** 2
									self.drawarea.cursor_x = tmp_x - tmp_x % 10
									self.drawarea.cursor_y = c[2] - c[2] % 10
							
					else:
						for p in c[1].input_pins + c[1].output_pins:
							if min_dist > (tmp_x - c[1].pos_x - p[0]) ** 2 + (tmp_y - c[1].pos_y - p[1]) ** 2:
								min_dist = (tmp_x - c[1].pos_x - p[0]) ** 2 + (tmp_y - c[1].pos_y - p[1]) ** 2
								self.drawarea.cursor_x = c[1].pos_x + p[0]
								self.drawarea.cursor_y = c[1].pos_y + p[1]
			
			if min_dist == 225:
				self.drawarea.cursor_x = tmp_x - tmp_x % 10
				self.drawarea.cursor_y = tmp_y - tmp_y % 10
			if oldcursor_x != self.drawarea.cursor_x or oldcursor_y != self.drawarea.cursor_y:
				self.drawarea.Refresh()
				
		else:
			if self.mouse_down and self.drag_enabled:
				# Move components
				ddx = self.px + self.drawarea.scrollX - self.drawarea.cursor_smooth_x
				ddy = self.py + self.drawarea.scrollY - self.drawarea.cursor_smooth_y
				old_delta_x = self.drag_delta_x
				old_delta_y = self.drag_delta_y
				if ddx >= 0:
					self.drag_delta_x = ddx - ddx % 10
				else:
					self.drag_delta_x = ddx + (-ddx % 10)
				if ddy >= 0:
					self.drag_delta_y = ddy - ddy % 10
				else:
					self.drag_delta_y = ddy + (-ddy % 10)
				if old_delta_x != self.drag_delta_x or old_delta_y != self.drag_delta_y:
					for c in self.circuit.selected_components:
						if c[0] == const.component_net:
							c[1] += self.drag_delta_x - old_delta_x
							c[2] += self.drag_delta_y - old_delta_y
							c[3] += self.drag_delta_x - old_delta_x
							c[4] += self.drag_delta_y - old_delta_y
							if old_delta_x == 0 and old_delta_y == 0:
								self.circuit.connect_nets(c[1] - (self.drag_delta_x - old_delta_x), c[2] - (self.drag_delta_y - old_delta_y), lock_selected = True)
								self.circuit.connect_nets(c[3] - (self.drag_delta_x - old_delta_x), c[4] - (self.drag_delta_y - old_delta_y), lock_selected = True)
						else:
							c[1].pos_x += self.drag_delta_x - old_delta_x
							c[1].pos_y += self.drag_delta_y - old_delta_y
							if old_delta_x == 0 and old_delta_y == 0:
								for p in c[1].input_pins + c[1].output_pins:
									self.circuit.connect_nets(c[1].pos_x - (self.drag_delta_x - old_delta_x) + p[0], c[1].pos_y - (self.drag_delta_y - old_delta_y) + p[1], lock_selected = True)
								
					if not self.component_dragged:
						self.drawarea.redraw = True
						
					self.component_dragged = True
					self.drawarea.Refresh()
					
			else:
				self.drawarea.cursor_smooth_x = self.px + self.drawarea.scrollX
				self.drawarea.cursor_smooth_y = self.py + self.drawarea.scrollY
				
				if self.rect_select_enabled:
					self.drawarea.Refresh()
				else:
					# Highlight component
					old_nearest_component = self.nearest_component
					self.nearest_component = None
					for c in self.circuit.components:
						if c[0] == const.component_net:
							if (c[1] - 3 <= self.drawarea.cursor_smooth_x <= c[3] + 3 or c[3] - 3 <= self.drawarea.cursor_smooth_x <= c[1] + 3) and (c[2] - 3 <= self.drawarea.cursor_smooth_y <= c[4] + 3 or c[4] - 3 <= self.drawarea.cursor_smooth_y <= c[2] + 3):
								if ((c[4] - c[2]) * (self.drawarea.cursor_smooth_x - c[1]) + (c[3] - c[1]) * (self.drawarea.cursor_smooth_y - c[2])) ** 2 / ((c[3] - c[1]) ** 2 + (c[4] - c[2]) ** 2) <= 9:
									self.nearest_component = c
									break
						else:
							if c[1].isMouseOvered(self.drawarea.cursor_smooth_x, self.drawarea.cursor_smooth_y):
								self.nearest_component = c
								break
								
					if old_nearest_component is not self.nearest_component:
						self.drawarea.Refresh()
			
